require 'yourapi/configuration'
require 'yourapi/client/errors'
require 'httparty'

module Yourapi
  class Client
    include HTTParty

    ACCEPT_HEADER = 'Accept'
    USER_ID_HEADER = 'X-Matchpoint-User-ID'.freeze
    APPLICATION_ID_HEADER = 'X-Matchpoint-Application-ID'.freeze
    APPLICATION_SECRET_KEY_HEADER = 'X-Matchpoint-Application-Secret-Key'.freeze

    def initialize(user_id)
      @application_id = Yourapi.configuration.application_id
      @application_secret_key = Yourapi.configuration.application_secret_key
      @base_uri = Yourapi.configuration.base_uri
      @user_id = user_id

      self.class.base_uri @base_uri
    end

    def authenticate(options = {})
      request(:get, '/v2/users/me', options)['user']
    end

    def create_transaction(user_id, value, options = {})
      request(:post, "/v2/users/#{user_id}/transactions", options.merge(query: { value: value }))['transaction']
    end

    def cancel_transaction(user_id, id, options = {})
      request(:delete, "/v2/users/#{user_id}/transactions/#{id}", options)
    end

    def notify(user_id, params = {}, options = {})
      request(:post, "/v2/users/#{user_id}/notifications", options.merge(body: params))
    end

    def request(method, path, options = {})
      post_process(self.class.send(method, path, options.merge(headers: headers)))
    end

    protected

    def post_process(response)
      raise AuthenticationFailed.new if response.code == 401
      raise ApiError.new(response['error']) if response.code >= 400

      response.parsed_response
    end

    def headers
      {
        ACCEPT_HEADER => 'application/json',
        USER_ID_HEADER => @api_key,
        APPLICATION_ID_HEADER => @application_id,
        APPLICATION_SECRET_KEY_HEADER => @application_secret_key
      }.keep_if { |k, v| v }
    end
  end
end
